@@ -1,26 +1,23 @@
-/* radare2 - LGPL - Copyright 2009-2024 - nibble, pancake, xvilka */
+/* radare2 - LGPL - Copyright 2009-2025 - nibble, pancake, xvilka */
 
 #ifndef R2_ANAL_H
 #define R2_ANAL_H
 
-/* use old refs and function storage */
-// still required by core in lot of places
-#define USE_VARSUBS 0
-
 #include <r_cons.h>
-#include <r_types.h>
 #include <r_io.h>
 #include <r_esil.h>
 #include <r_arch.h>
 #include <r_list.h>
 #include <r_util/r_print.h>
 #include <r_search.h>
-#include <r_util.h>
 #include <r_bind.h>
 #include <r_syscall.h>
 #include <r_flag.h>
 #include <r_bin.h>
 #include <r_codemeta.h>
+#include <r_anal/hint.h>
+#include <r_vec.h>
+#include <r_util/r_intervaltree.h>
 #include <sdb/set.h>
 
 #ifdef __cplusplus
@@ -63,6 +60,15 @@ typedef struct r_anal_range_t {
 	RBNode rb;
 } RAnalRange;
 
+enum {
+	R_ANAL_REFLINE_TYPE_UTF8 = 1,
+	R_ANAL_REFLINE_TYPE_WIDE = 2,  /* reflines have a space between them */
+	R_ANAL_REFLINE_TYPE_MIDDLE_BEFORE = 4, /* do not consider starts/ends of
+	                                        * reflines (used for comment lines before disasm) */
+	R_ANAL_REFLINE_TYPE_MIDDLE_AFTER = 8, /* as above but for lines after disasm */
+	R_ANAL_REFLINE_TYPE_SPLIT = 16 /* use reflines2 for upward lines */
+};
+
 enum {
 	R_ANAL_DATA_TYPE_NULL = 0,
 	R_ANAL_DATA_TYPE_UNKNOWN = 1,
@@ -217,16 +223,34 @@ typedef enum {
 	R_ANAL_BASE_TYPE_KIND_ATOMIC, // For real atomic base types
 } RAnalBaseTypeKind;
 
+static inline void anal_struct_member_fini(RAnalStructMember *member) {
+	free (member->name);
+	free (member->type);
+}
+
+static inline void anal_union_member_fini(RAnalUnionMember *member) {
+	free (member->name);
+	free (member->type);
+}
+
+static inline void anal_enum_case_fini(RAnalEnumCase *cas) {
+	free (cas->name);
+}
+
+R_VEC_TYPE_WITH_FINI (RVecAnalStructMember, RAnalStructMember, anal_struct_member_fini);
+R_VEC_TYPE_WITH_FINI (RVecAnalUnionMember, RAnalUnionMember, anal_union_member_fini);
+R_VEC_TYPE_WITH_FINI (RVecAnalEnumCase, RAnalEnumCase, anal_enum_case_fini);
+
 typedef struct r_anal_base_type_struct_t {
-	RVector/*<RAnalStructMember>*/ members;
+	RVecAnalStructMember members;
 } RAnalBaseTypeStruct;
 
 typedef struct r_anal_base_type_union_t {
-	RVector/*<RAnalUnionMember>*/ members;
+	RVecAnalUnionMember members;
 } RAnalBaseTypeUnion;
 
 typedef struct r_anal_base_type_enum_t {
-	RVector/*<RAnalEnumCase*/ cases; // list of all the enum casessssss
+	RVecAnalEnumCase cases; // list of all the enum casessssss
 } RAnalBaseTypeEnum;
 
 typedef struct r_anal_base_type_t {
@@ -255,6 +279,8 @@ struct r_anal_attr_t {
 	RAnalAttr *next;
 };
 
+typedef struct r_anal_var_t RAnalVar;
+
 /* Stores useful function metadata */
 typedef struct r_anal_function_meta_t {
 	// _min and _max are calculated lazily when queried.
@@ -267,18 +293,20 @@ typedef struct r_anal_function_meta_t {
 	int numcallrefs;    // number of calls
 } RAnalFcnMeta;
 
+R_VEC_TYPE (RVecAnalVarPtr, RAnalVar *);
+
 typedef struct r_anal_function_t {
 	// TODO R2_600 Use RBinName here
 	char *name;
 	char *realname; // R2_590: add realname for the mangled one
 	int bits; // ((> bits 0) (set-bits bits))
 	int type;
-	const char *cc; // calling convention, should come from RAnal.constpool
+	const char *callconv; // calling convention, should come from RAnal.constpool
 	ut64 addr;
 	HtUP/*<ut64, char *>*/ *labels;
 	HtPP/*<char *, ut64 *>*/ *label_addrs;
-	RPVector vars;
-	HtUP/*<st64, RPVector<RAnalVar *>>*/ *inst_vars; // offset of instructions => the variables they access
+	RVecAnalVarPtr vars;
+	HtUP/*<st64, RVecAnalVarPtr *>*/ *inst_vars; // offset of instructions => the variables they access
 	ut64 reg_save_area; // size of stack area pre-reserved for saving registers
 	st64 bp_off; // offset of bp inside owned stack frame
 	st64 stack;  // stack frame size
@@ -293,13 +321,11 @@ typedef struct r_anal_function_t {
 	ut8 *fingerprint; // TODO: make is fuzzy and smarter
 	size_t fingerprint_size;
 	RAnalDiff *diff;
-	RList *bbs; // TODO: should be RPVector
+	RList *bbs; // TODO: should be RVec
 	RAnalFcnMeta meta;
 	RList *imports; // maybe bound to class?
 	struct r_anal_t *anal; // this function is associated with this instance
-#if R2_600
 	ut64 ts; // timestamp when the function was registered. useful to sort them by order or "incremental projects"). afla
-#endif
 } RAnalFunction;
 
 typedef struct r_anal_func_arg_t {
@@ -336,9 +362,8 @@ typedef enum {
 
 /* meta */
 typedef struct r_anal_meta_item_t {
-	// XXX missing size
-	RAnalMetaType type;
-	int subtype;
+	uint8_t type; // RAnalMetaType type;
+	uint8_t subtype;
 	char *str;
 	const RSpace *space;
 } RAnalMetaItem;
@@ -388,12 +413,14 @@ typedef struct r_anal_options_t {
 	bool propagate_noreturn;
 	bool recursive_noreturn; // anal.rnr
 	bool slow;
-#if R2_USE_NEW_ABI
 	bool flagends;
 	bool zigndups;
 	bool icods; // R2_600 -- add anal.icods or anal.xrefs.indirect references. needed for stm8 at least
-	// R2_600 - add zign_dups field for "zign.dups" config
-#endif
+	char *tparser;
+	bool dynprefix;
+	const char *defprefix;
+	const char *prefix_marker;
+	ut64 prefix_radius;
 } RAnalOptions;
 
 // XXX we have cc / calling conventions / abi settings already no need for a custom enum here
@@ -407,6 +434,8 @@ typedef struct r_anal_hint_cb_t {
 	void (*on_bits) (struct r_anal_t *a, ut64 addr, int bits, bool set);
 } RHintCb;
 
+R_VEC_TYPE (RVecAnalAddrHintRecord, RAnalAddrHintRecord);
+
 typedef struct r_anal_thread_t {
 	int id;
 	int map; // tls map id
@@ -435,7 +464,7 @@ typedef struct r_anal_t {
 	ut8 *last_disasm_reg;
 	int last_disasm_reg_size;
 	RSyscall *syscall;
-	int diff_ops;
+	bool diff_ops;
 	double diff_thbb;
 	double diff_thfcn;
 	RIOBind iob;
@@ -463,7 +492,7 @@ typedef struct r_anal_t {
 	//moved from RAnalFcn
 	Sdb *sdb; // root
 	Sdb *sdb_pins;
-	HtUP/*<RVector<RAnalAddrHintRecord>>*/ *addr_hints; // all hints that correspond to a single address
+	HtUP/*<RVecAnalAddrHintRecord>*/ *addr_hints; // all hints that correspond to a single address
 	RBTree/*<RAnalArchHintRecord>*/ arch_hints;
 	RBTree/*<RAnalArchBitsRecord>*/ bits_hints;
 	RHintCb hint_cbs;
@@ -475,6 +504,7 @@ typedef struct r_anal_t {
 	RAnalCallbacks cb;
 	RAnalOptions opt;
 	RList *reflines;
+	RList *reflines2;
 	RListComparator columnSort;
 	int stackptr;
 	bool (*log)(struct r_anal_t *anal, const char *msg);
@@ -490,6 +520,7 @@ typedef struct r_anal_t {
 	RAnalBacktraces btstore;
 	/* private */
 	RThreadLock *lock;
+	RList *eligible;
 	ut64 cmpval;
 	ut64 lea_jmptbl_ip;
 	int cs_obits;
@@ -529,12 +560,15 @@ typedef struct r_anal_var_access_t {
 } RAnalVarAccess;
 
 typedef struct r_anal_var_constraint_t {
-	_RAnalCond cond;
+	RAnalCondType cond;
 	ut64 val;
 } RAnalVarConstraint;
 
+R_VEC_TYPE (RVecAnalVarAccess, RAnalVarAccess);
+R_VEC_TYPE (RVecAnalVarConstraint, RAnalVarConstraint);
+
 // generic for args and locals
-typedef struct r_anal_var_t {
+struct r_anal_var_t {
 	RAnalFunction *fcn;
 	char *name; // name of the variable
 	char *type; // cparse type of the variable
@@ -542,13 +576,13 @@ typedef struct r_anal_var_t {
 	bool isarg;
 	int delta;   /* delta offset inside stack frame */
 	char *regname; // name of the register
-	RVector/*<RAnalVarAccess>*/ accesses; // ordered by offset, touch this only through API or expect uaf
+	RVecAnalVarAccess accesses; // ordered by offset, touch this only through API or expect uaf
 	char *comment;
-	RVector/*<RAnalVarConstraint>*/ constraints;
+	RVecAnalVarConstraint constraints;
 
 	// below members are just for caching, TODO: remove them and do it better
 	int argnum;
-} RAnalVar;
+};
 
 // RAnalVar "prototype", RAnalVar w/o function used for serialization
 typedef struct r_anal_var_proto_t {
@@ -559,6 +593,8 @@ typedef struct r_anal_var_proto_t {
 	int delta;
 } RAnalVarProt;
 
+R_VEC_TYPE (RVecIntervalNodePtr, RIntervalNode *);
+
 // Refers to a variable or a struct field inside a variable, only for varsub
 R_DEPRECATE typedef struct r_anal_var_field_t {
 	char *name;
@@ -596,11 +632,12 @@ typedef struct r_anal_bind_t {
 	RAnalUse use;
 } RAnalBind;
 
-#define R_ANAL_COND_SINGLE(x) (!x->arg[1] || x->arg[0]==x->arg[1])
+#define R_ANAL_CONDTYPE_SINGLE(x) (!x->right || x->left==x->right)
 
 typedef struct r_anal_cond_t {
 	int type; // filled by CJMP opcode
-	RArchValue *arg[2]; // filled by CMP opcode
+	RArchValue *left; // filled by CMP left opcode
+	RArchValue *right; // filled by CMP right opcode
 } RAnalCond;
 
 typedef struct r_anal_bb_t {
@@ -642,6 +679,7 @@ typedef struct r_anal_bb_t {
 } RAnalBlock;
 
 typedef enum {
+	R_ANAL_REF_TYPE_ANY = -1,   // match any type
 	R_ANAL_REF_TYPE_NULL = 0,   // unknown/undefined
 	R_ANAL_REF_TYPE_ERROR = 1,  // unreadable/invalid
 	R_ANAL_REF_TYPE_CODE = 'c', // code ref
@@ -723,6 +761,26 @@ typedef struct r_anal_esil_cfg_t {
 	RGraph *g;
 } RAnalEsilCFG;
 
+// this is 80-bit offsets so we can address every piece of esil in an instruction
+typedef struct r_anal_esil_expr_offset_t {
+	ut64 off;
+	ut16 idx;
+} RAnalEsilEOffset;
+
+typedef enum {
+	R_ANAL_ESIL_BLOCK_ENTER_NORMAL = 0,
+	R_ANAL_ESIL_BLOCK_ENTER_TRUE,
+	R_ANAL_ESIL_BLOCK_ENTER_FALSE,
+	R_ANAL_ESIL_BLOCK_ENTER_GLUE,
+} RAnalEsilBlockEnterType;
+
+typedef struct r_anal_esil_basic_block_t {
+	RAnalEsilEOffset first;
+	RAnalEsilEOffset last;
+	char *expr;	//synthesized esil-expression for this block
+	RAnalEsilBlockEnterType enter;	//maybe more type is needed here
+} RAnalEsilBB;
+
 enum {
 	R_ANAL_ESIL_DFG_TAG_CONST = 1,
 	R_ANAL_ESIL_DFG_TAG_VAR = 2,
@@ -762,9 +820,16 @@ typedef struct r_anal_esil_dfg_node_t {
 
 typedef bool (*RAnalCmdCallback)(/* Rcore */RAnal *anal, const char* input);
 
+typedef struct {
+	bool jmptbl_found;
+} RAnalOpFlow;
+
 typedef int (*RAnalOpCallback)(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask);
+typedef RAnalOpFlow (*RAnalOpFlowCallback)(RAnal *a, RAnalOp *op);
 typedef int (*RAnalOpAsmCallback)(RAnal *a, ut64 addr, const char *str, ut8 *outbuf, int outlen);
 
+typedef bool (*RAnalPluginEligible)(RAnal *a);
+
 typedef bool (*RAnalRegProfCallback)(RAnal *a);
 typedef char*(*RAnalRegProfGetCallback)(RAnal *a);
 typedef int (*RAnalFPBBCallback)(RAnal *a, RAnalBlock *bb);
@@ -777,6 +842,10 @@ typedef int (*REsilCB)(REsil *esil);
 typedef int (*REsilLoopCB)(REsil *esil, RAnalOp *op);
 typedef int (*REsilTrapCB)(REsil *esil, int trap_type, int trap_code);
 
+typedef char *(*RAnalTypesParser)(RAnal *a, const char *s);
+typedef char *(*RAnalTypesParserFile)(RAnal *a, const char *s, const char *dir);
+typedef char *(*RAnalTypesParserText)(RAnal *a, const char *s);
+
 typedef struct r_anal_plugin_t {
 	RPluginMeta meta;
 
@@ -784,10 +853,17 @@ typedef struct r_anal_plugin_t {
 
 	bool (*init)(RAnal *a);
 	bool (*fini)(RAnal *a);
+	RAnalPluginEligible eligible;
 
 	// legacy r_anal_functions
 	RAnalOpCallback op;
 	RAnalCmdCallback cmd;
+	RAnalOpFlowCallback opflow;
+
+	// implement custom types parser and dumper
+	RAnalTypesParserText tparse_text;
+	RAnalTypesParserFile tparse_file;
+	RAnalTypesParser tdump;
 #if 1
 	/// XXX unused but referenced, maybe worth checking in case we want them for anal
 	RAnalFPBBCallback fingerprint_bb;
@@ -898,7 +974,7 @@ R_API bool r_anal_block_recurse_depth_first(RAnalBlock *block, RAnalBlockCb cb,
 R_API RList *r_anal_block_recurse_list(RAnalBlock *block);
 
 // return one shortest path from block to dst or NULL if none exists.
-R_API R_NULLABLE RList/*<RAnalBlock *>*/ *r_anal_block_shortest_path(RAnalBlock *block, ut64 dst);
+R_API RList/*<RAnalBlock *>*/ * R_NULLABLE r_anal_block_shortest_path(RAnalBlock *block, ut64 dst);
 
 // Add a case to the block's switch_op.
 // If block->switch_op is NULL, it will be created with the given switch_addr.
@@ -944,7 +1020,7 @@ R_API RList *r_anal_get_functions_in(RAnal *anal, ut64 addr);
 // returns the function that has its entrypoint at addr or NULL
 R_API RAnalFunction *r_anal_get_function_at(RAnal *anal, ut64 addr);
 
-R_API bool r_anal_function_delete(RAnalFunction *fcn);
+R_API bool r_anal_function_delete(RAnal *anal, RAnalFunction *fcn);
 
 // rhange the entrypoint of fcn
 // This can fail (and return false) if there is already another function at the new address
@@ -957,7 +1033,6 @@ R_API bool r_anal_function_rename(RAnalFunction *fcn, const char *name);
 R_API void r_anal_function_add_block(RAnalFunction *fcn, RAnalBlock *bb);
 R_API void r_anal_function_remove_block(RAnalFunction *fcn, RAnalBlock *bb);
 
-
 // size of the entire range that the function spans, including holes.
 // this is exactly r_anal_function_max_addr() - r_anal_function_min_addr()
 R_API ut64 r_anal_function_linear_size(RAnalFunction *fcn);
@@ -968,11 +1043,13 @@ R_API ut64 r_anal_function_min_addr(RAnalFunction *fcn);
 // first address directly after the function
 R_API ut64 r_anal_function_max_addr(RAnalFunction *fcn);
 
+
+
 // size from the function entrypoint (fcn->addr) to the end of the function (r_anal_function_max_addr)
 R_API ut64 r_anal_function_size_from_entry(RAnalFunction *fcn);
 
 // the "real" size of the function, that is the sum of the size of the
-// basicblocks this function is composed of
+// basicblocks this function is composed of -- i think it should be just _size()
 R_API ut64 r_anal_function_realsize(const RAnalFunction *fcn);
 
 // returns whether the function contains a basic block that contains addr
@@ -1006,6 +1083,7 @@ R_API const char *r_anal_functiontype_tostring(int type);
 R_API int r_anal_function_coverage(RAnalFunction *fcn);
 R_API int r_anal_function_bb(RAnal *anal, RAnalFunction *fcn, ut64 addr, int depth);
 R_API void r_anal_bind(RAnal *b, RAnalBind *bnd);
+R_API void r_anal_type_match(RAnal *anal, RAnalFunction *fcn);
 R_API bool r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int bits);
 R_API void r_anal_add_import(RAnal *anal, const char *imp);
 R_API void r_anal_remove_import(RAnal *anal, const char *imp);
@@ -1043,15 +1121,15 @@ R_API void r_anal_pin_unset(RAnal *a, ut64 addr);
 
 /* fcn.c */
 R_API ut32 r_anal_function_cost(RAnalFunction *fcn);
-R_API int r_anal_function_count_edges(const RAnalFunction *fcn, R_NULLABLE int *ebbs);
+R_API int r_anal_function_count_edges(const RAnalFunction *fcn, int * R_NULLABLE ebbs);
 
 R_API RAnalFunction *r_anal_get_function_byname(RAnal *anal, const char *name);
 
-R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype);
+R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, int reftype);
 R_API int r_anal_function_del(RAnal *anal, ut64 addr);
 R_API bool r_anal_function_add_bb(RAnal *anal, RAnalFunction *fcn,
 		ut64 addr, ut64 size,
-		ut64 jump, ut64 fail, R_BORROW RAnalDiff *diff);
+		ut64 jump, ut64 fail, R_UNOWNED RAnalDiff *diff);
 R_API bool r_anal_check_fcn(RAnal *anal, ut8 *buf, ut16 bufsz, ut64 addr, ut64 low, ut64 high);
 
 R_API void r_anal_function_check_bp_use(RAnalFunction *fcn);
@@ -1060,7 +1138,6 @@ R_API void r_anal_function_update_analysis(RAnalFunction *fcn);
 
 #define R_ANAL_FCN_VARKIND_LOCAL 'v'
 
-
 R_API int r_anal_function_var_del_byindex(RAnal *a, ut64 fna, const char kind, int scope, ut32 idx);
 /* args */
 R_API int r_anal_var_count(RAnal *a, RAnalFunction *fcn, int kind, int type);
@@ -1075,7 +1152,6 @@ R_API int r_anal_function_complexity(RAnalFunction *fcn);
 R_API int r_anal_function_loops(RAnalFunction *fcn);
 R_API void r_anal_trim_jmprefs(RAnal *anal, RAnalFunction *fcn);
 R_API void r_anal_del_jmprefs(RAnal *anal, RAnalFunction *fcn);
-R_API char *r_anal_function_get_json(RAnalFunction *function);
 R_API RAnalFunction *r_anal_function_next(RAnal *anal, ut64 addr);
 R_API char *r_anal_function_get_signature(RAnalFunction *function);
 R_API bool r_anal_function_del_signature(RAnal *a, const char *name);
@@ -1090,6 +1166,7 @@ R_API int r_anal_function_instrcount(RAnalFunction *fcn);
 R_API bool r_anal_function_islineal(RAnalFunction *fcn);
 R_API const char *r_anal_pin_get(RAnal *a, const char *name);
 R_API const char *r_anal_pin_at(RAnal *a, ut64 addr);
+R_API const char * R_NONNULL r_anal_fcn_prefix_at(RAnal *anal, ut64 addr);
 R_API bool r_anal_pin_set(RAnal *a, const char *name, const char *cmd);
 
 typedef bool (* RAnalRefCmp)(RAnalRef *ref, void *data);
@@ -1110,12 +1187,14 @@ R_API RVecAnalRef *r_anal_xrefs_get(RAnal *anal, ut64 to);
 R_API RVecAnalRef *r_anal_refs_get(RAnal *anal, ut64 from);
 R_API bool r_anal_xrefs_has_xrefs_at(RAnal *anal, ut64 at);
 R_API RVecAnalRef *r_anal_xrefs_get_from(RAnal *anal, ut64 to);
-R_API void r_anal_xrefs_list(RAnal *anal, int rad, const char *arg);
+R_API void r_anal_xrefs_list(RAnal *anal, int rad, const char *arg, RTable *t);
 R_API ut64 r_anal_xrefs_count(RAnal *anal);
 R_API ut64 r_anal_xrefs_count_at(RAnal *anal, ut64 to);
 R_API RVecAnalRef *r_anal_function_get_refs(RAnalFunction *fcn);
 R_API RVecAnalRef *r_anal_function_get_all_xrefs(RAnalFunction *fcn);
 R_API RVecAnalRef *r_anal_function_get_xrefs(RAnalFunction *fcn);
+R_API ut64 r_anal_function_count_refs(RAnalFunction *fcn, RAnalRefType type);
+R_API ut64 r_anal_function_count_xrefs(RAnalFunction *fcn, RAnalRefType type);
 R_API bool r_anal_xrefs_set(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
 R_API bool r_anal_xref_del(RAnal *anal, ut64 from, ut64 to);
 
@@ -1126,14 +1205,14 @@ R_API void r_anal_remove_parsed_type(RAnal *anal, const char *name);
 R_API void r_anal_save_parsed_type(RAnal *anal, const char *parsed);
 
 /* var.c */
-R_API R_OWN char *r_anal_function_autoname_var(RAnalFunction *fcn, char kind, const char *pfx, int ptr);
-R_API R_BORROW RAnalVar *r_anal_function_set_var(RAnalFunction *fcn, int delta, char kind, R_NULLABLE const char *type, int size, bool isarg, R_NONNULL const char *name);
+R_API R_OWNED char *r_anal_function_autoname_var(RAnalFunction *fcn, char kind, const char *pfx, int ptr);
+R_API R_UNOWNED RAnalVar *r_anal_function_set_var(RAnalFunction *fcn, int delta, char kind, const char * R_NULLABLE type, int size, bool isarg, const char * R_NONNULL name);
 R_API bool r_anal_function_set_var_prot(RAnalFunction *fcn, RList /*<RAnalVarProt>*/ *l);
-R_API R_BORROW RAnalVar *r_anal_function_get_var(RAnalFunction *fcn, char kind, int delta);
+R_API R_UNOWNED RAnalVar *r_anal_function_get_var(RAnalFunction *fcn, char kind, int delta);
 R_API RList *r_anal_var_deserialize(const char *ser);
 R_API char *r_anal_var_prot_serialize(RList /*<RAnalVarProt>*/ *l, bool spaces);
 R_API RList /*<RAnalVarProt>*/ *r_anal_var_get_prots(RAnalFunction *fcn);
-R_API R_BORROW RAnalVar *r_anal_function_get_var_byname(RAnalFunction *fcn, const char *name);
+R_API R_UNOWNED RAnalVar *r_anal_function_get_var_byname(RAnalFunction *fcn, const char *name);
 R_API void r_anal_function_delete_vars_by_kind(RAnalFunction *fcn, RAnalVarKind kind);
 R_API void r_anal_function_delete_all_vars(RAnalFunction *fcn);
 R_API void r_anal_function_delete_unused_vars(RAnalFunction *fcn);
@@ -1141,16 +1220,16 @@ R_API void r_anal_function_delete_var(RAnalFunction *fcn, RAnalVar *var);
 R_API bool r_anal_function_rebase_vars(RAnal *a, RAnalFunction *fcn);
 R_API st64 r_anal_function_get_var_stackptr_at(RAnalFunction *fcn, st64 delta, ut64 addr);
 R_API const char *r_anal_function_get_var_reg_at(RAnalFunction *fcn, st64 delta, ut64 addr);
-R_API R_BORROW RPVector *r_anal_function_get_vars_used_at(RAnalFunction *fcn, ut64 op_addr);
+R_API R_UNOWNED RVecAnalVarPtr *r_anal_function_get_vars_used_at(RAnalFunction *fcn, ut64 op_addr);
 
-R_API bool r_anal_var_rename(RAnalVar *var, const char *new_name, bool verbose);
-R_API void r_anal_var_set_type(RAnalVar *var, const char *type);
-R_API void r_anal_var_delete(RAnalVar *var);
+R_API bool r_anal_var_rename(RAnal *anal, RAnalVar *var, const char *new_name);
+R_API void r_anal_var_set_type(RAnal *anal, RAnalVar *var, const char *type);
+R_API bool r_anal_var_delete(RAnal *anal, RAnalVar *var);
 R_API ut64 r_anal_var_addr(RAnalVar *var);
-R_API void r_anal_var_set_access(RAnalVar *var, const char *reg, ut64 access_addr, int access_type, st64 stackptr);
+R_API bool r_anal_var_set_access(RAnal *anal, RAnalVar *var, const char *reg, ut64 access_addr, int access_type, st64 stackptr);
 R_API void r_anal_var_remove_access_at(RAnalVar *var, ut64 address);
 R_API void r_anal_var_clear_accesses(RAnalVar *var);
-R_API void r_anal_var_add_constraint(RAnalVar *var, R_BORROW RAnalVarConstraint *constraint);
+R_API void r_anal_var_add_constraint(RAnalVar *var, R_UNOWNED RAnalVarConstraint *constraint);
 R_API char *r_anal_var_get_constraints_readable(RAnalVar *var);
 
 // Get the access to var at exactly addr if there is one
@@ -1171,12 +1250,12 @@ typedef struct r_anal_function_vars_cache {
 	RList *rvars;
 	RList *svars;
 } RAnalFcnVarsCache;
+
 R_API void r_anal_function_vars_cache_init(RAnal *anal, RAnalFcnVarsCache *cache, RAnalFunction *fcn);
 R_API void r_anal_function_vars_cache_fini(RAnalFcnVarsCache *cache);
 
-R_API char *r_anal_function_format_sig(R_NONNULL RAnal *anal, R_NONNULL RAnalFunction *fcn, R_NULLABLE char *fcn_name,
-		R_NULLABLE RAnalFcnVarsCache *reuse_cache, R_NULLABLE const char *fcn_name_pre, R_NULLABLE const char *fcn_name_post);
-
+R_API char *r_anal_function_format_sig(RAnal *anal, RAnalFunction *fcn, char * R_NULLABLE fcn_name,
+		RAnalFcnVarsCache * R_NULLABLE reuse_cache, const char * R_NULLABLE fcn_name_pre, const char * R_NULLABLE fcn_name_post);
 
 /* project */
 #define R_ANAL_THRESHOLDFCN 0.7F
@@ -1184,14 +1263,13 @@ R_API char *r_anal_function_format_sig(R_NONNULL RAnal *anal, R_NONNULL RAnalFun
 
 /* diff.c */
 R_API RAnalDiff *r_anal_diff_new(void);
-R_API void r_anal_diff_setup(RAnal *anal, int doops, double thbb, double thfcn);
-R_API void r_anal_diff_setup_i(RAnal *anal, int doops, int thbb, int thfcn);
+R_API void r_anal_diff_setup(RAnal *anal, bool doops, double thbb, double thfcn);
+R_API void r_anal_diff_setup_i(RAnal *anal, bool doops, int thbb, int thfcn);
 R_API void r_anal_diff_free(RAnalDiff *diff);
 R_API int r_anal_diff_fingerprint_bb(RAnal *anal, RAnalBlock *bb);
 R_API size_t r_anal_diff_fingerprint_fcn(RAnal *anal, RAnalFunction *fcn);
 R_API bool r_anal_diff_bb(RAnal *anal, RAnalFunction *fcn, RAnalFunction *fcn2);
 R_API int r_anal_diff_fcn(RAnal *anal, RList *fcns, RList *fcns2);
-R_API int r_anal_diff_eval(RAnal *anal);
 
 R_API RAnalCond *r_anal_cond_new(void);
 R_API RAnalCond *r_anal_cond_new_from_op(RAnalOp *op);
@@ -1201,20 +1279,22 @@ R_API char *r_anal_cond_tostring(RAnalCond *cond);
 R_API int r_anal_cond_eval(RAnal *anal, RAnalCond *cond);
 R_API RAnalCond *r_anal_cond_new_from_string(const char *str);
 R_API const char *r_anal_cond_type_tostring(int cc);
+R_API const char *r_anal_cond_typeexpr_tostring(int cc);
 
 /* jmptbl */
 R_API bool r_anal_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, ut64 jmpaddr, ut64 table, ut64 tablesize, ut64 default_addr);
+R_API void r_anal_jmptbl_list(RAnal *anal, RAnalFunction *fcn, RAnalBlock *bb, ut64 saddr, ut64 jaddr, RList *cases, int loadsz);
 
 // TODO: should be renamed
 R_API bool try_get_delta_jmptbl_info(RAnal *a, RAnalFunction *fcn, ut64 jmp_addr, ut64 lea_addr, ut64 *table_size, ut64 *default_case, st64 *start_casenum_shift);
-R_API bool try_walkthrough_jmptbl(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
+R_API bool r_anal_jmptbl_walk(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
 R_API bool try_walkthrough_casetbl(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 casetbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
 R_API bool try_get_jmptbl_info(RAnal *analysis, RAnalFunction *fcn, ut64 addr, RAnalBlock *my_bb, ut64 *table_size, ut64 *default_case, st64 *start_casenum_shift);
 R_API int walkthrough_arm_jmptbl_style(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, ut64 jmptbl_loc, ut64 sz, ut64 jmptbl_size, ut64 default_case, int ret0);
 
 /* reflines.c */
 R_API RList* /*<RAnalRefline>*/ r_anal_reflines_get(RAnal *anal,
-		ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall);
+		ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall, int splitmode);
 R_API int r_anal_reflines_middle(RAnal *anal, RList *list, ut64 addr, int len);
 R_API RAnalRefStr *r_anal_reflines_str(void *core, ut64 addr, int opts);
 R_API void r_anal_reflines_str_free(RAnalRefStr *refstr);
@@ -1301,20 +1381,20 @@ R_API const char *r_meta_get_string(RAnal *a, RAnalMetaType type, ut64 addr);
 R_API void r_meta_set_data_at(RAnal *a, ut64 addr, ut64 wordsz);
 
 // Returns the item with given type that starts at addr in the current space or NULL. The size of this item  optionally returned through size.
-R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT R_NULLABLE ut64 *size);
+R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT ut64 * R_NULLABLE size);
 
 // Returns the node for one meta item with the given type that contains addr in the current space or NULL.
 // To get all the nodes, use r_meta_get_all_in().
 R_API RIntervalNode *r_meta_get_in(RAnal *a, ut64 addr, RAnalMetaType type);
 
 // Returns all nodes for items starting at the given address in the current space.
-R_API RPVector/*<RIntervalNode<RMetaItem> *>*/ *r_meta_get_all_at(RAnal *a, ut64 at);
+R_API RVecIntervalNodePtr *r_meta_get_all_at(RAnal *a, ut64 at);
 
 // Returns all nodes for items with the given type containing the given address in the current space.
-R_API RPVector/*<RIntervalNode<RMetaItem> *>*/ *r_meta_get_all_in(RAnal *a, ut64 at, RAnalMetaType type);
+R_API RVecIntervalNodePtr *r_meta_get_all_in(RAnal *a, ut64 at, RAnalMetaType type);
 
 // Returns all nodes for items with the given type intersecting the given interval in the current space.
-R_API RPVector/*<RIntervalNode<RMetaItem> *>*/ *r_meta_get_all_intersect(RAnal *a, ut64 start, ut64 size, RAnalMetaType type);
+R_API RVecIntervalNodePtr *r_meta_get_all_intersect(RAnal *a, ut64 start, ut64 size, RAnalMetaType type);
 
 // Delete all meta items in the given space
 R_API void r_meta_space_unset_for(RAnal *a, const RSpace *space);
@@ -1329,10 +1409,10 @@ R_API void r_meta_rebase(RAnal *anal, ut64 diff);
 R_API ut64 r_meta_get_size(RAnal *a, RAnalMetaType type);
 
 R_API const char *r_meta_type_tostring(int type);
-R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, bool show_full);
-R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq);
-R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq);
-R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq);
+R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, RTable *t, bool show_full);
+R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq, RTable *t);
+R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq, RTable *t);
+R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq, RTable *t);
 
 /* hints */
 
@@ -1355,7 +1435,7 @@ R_API void r_anal_hint_set_ret(RAnal *a, ut64 addr, ut64 val);
 R_API void r_anal_hint_set_high(RAnal *a, ut64 addr);
 R_API void r_anal_hint_set_stackframe(RAnal *a, ut64 addr, ut64 size);
 R_API void r_anal_hint_set_val(RAnal *a, ut64 addr, ut64 v);
-R_API void r_anal_hint_set_arch(RAnal *a, ut64 addr, R_NULLABLE const char *arch); // arch == NULL => use global default
+R_API void r_anal_hint_set_arch(RAnal *a, ut64 addr, const char * R_NULLABLE arch); // arch == NULL => use global default
 R_API void r_anal_hint_set_bits(RAnal *a, ut64 addr, int bits); // bits == NULL => use global default
 R_API void r_anal_hint_unset_val(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_high(RAnal *a, ut64 addr);
@@ -1375,10 +1455,10 @@ R_API void r_anal_hint_unset_newbits(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_stackframe(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_arch(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_bits(RAnal *a, ut64 addr);
-R_API R_NULLABLE const RVector/*<const RAnalAddrHintRecord>*/ *r_anal_addr_hints_at(RAnal *anal, ut64 addr);
-typedef bool (*RAnalAddrHintRecordsCb)(ut64 addr, const RVector/*<const RAnalAddrHintRecord>*/ *records, void *user);
+R_API const RVecAnalAddrHintRecord * R_NULLABLE r_anal_addr_hints_at(RAnal * R_NONNULL anal, ut64 addr);
+typedef bool (*RAnalAddrHintRecordsCb)(ut64 addr, const RVecAnalAddrHintRecord *records, void *user);
 R_API void r_anal_addr_hints_foreach(RAnal *anal, RAnalAddrHintRecordsCb cb, void *user);
-typedef bool (*RAnalArchHintCb)(ut64 addr, R_NULLABLE const char *arch, void *user);
+typedef bool (*RAnalArchHintCb)(ut64 addr, const char * R_NULLABLE arch, void *user);
 R_API void r_anal_arch_hints_foreach(RAnal *anal, RAnalArchHintCb cb, void *user);
 typedef bool (*RAnalBitsHintCb)(ut64 addr, int bits, void *user);
 R_API void r_anal_bits_hints_foreach(RAnal *anal, RAnalBitsHintCb cb, void *user);
@@ -1386,12 +1466,12 @@ R_API void r_anal_bits_hints_foreach(RAnal *anal, RAnalBitsHintCb cb, void *user
 // get the hint-specified arch value to be considered at addr
 // hint_addr will optionally be set to the address where the hint that specifies this arch is placed or UT64_MAX
 // if there is no hint affecting addr.
-R_API R_NULLABLE R_BORROW const char *r_anal_hint_arch_at(RAnal *anal, ut64 addr, R_NULLABLE ut64 *hint_addr);
+R_API R_UNOWNED const char * R_NULLABLE  r_anal_hint_arch_at(RAnal *anal, ut64 addr, ut64 * R_NULLABLE hint_addr);
 
 // get the hint-specified bits value to be considered at addr
 // hint_addr will optionally be set to the address where the hint that specifies this arch is placed or UT64_MAX
 // if there is no hint affecting addr.
-R_API int r_anal_hint_bits_at(RAnal *anal, ut64 addr, R_NULLABLE ut64 *hint_addr);
+R_API int r_anal_hint_bits_at(RAnal *anal, ut64 addr, ut64 * R_NULLABLE hint_addr);
 
 R_API RAnalHint *r_anal_hint_get(RAnal *anal, ut64 addr); // accumulate all available hints affecting the given address
 
@@ -1426,6 +1506,11 @@ R_API void r_sign_space_unset_for(RAnal *a, const RSpace *space);
 R_API void r_sign_space_rename_for(RAnal *a, const RSpace *space, const char *oname, const char *nname);
 
 /* vtables */
+typedef struct vtable_method_info_t {
+	ut64 addr;           // addr of the function
+	ut64 vtable_offset;  // offset inside the vtable
+} RVTableMethodInfo;
+
 typedef struct {
 	RAnal *anal;
 	RAnalCPPABI abi;
@@ -1433,22 +1518,19 @@ typedef struct {
 	bool (*read_addr) (RAnal *anal, ut64 addr, ut64 *buf);
 } RVTableContext;
 
+R_VEC_TYPE (RVecRVTableMethodInfo, RVTableMethodInfo);
+
 typedef struct vtable_info_t {
 	ut64 saddr; //starting address
-	RVector methods;
+	RVecRVTableMethodInfo methods;
 } RVTableInfo;
 
-typedef struct vtable_method_info_t {
-	ut64 addr;           // addr of the function
-	ut64 vtable_offset;  // offset inside the vtable
-} RVTableMethodInfo;
-
 R_API void r_anal_vtable_info_free(RVTableInfo *vtable);
 R_API ut64 r_anal_vtable_info_get_size(RVTableContext *context, RVTableInfo *vtable);
 R_API bool r_anal_vtable_begin(RAnal *anal, RVTableContext *context);
 R_API RVTableInfo *r_anal_vtable_parse_at(RVTableContext *context, ut64 addr);
 R_API RList *r_anal_vtable_search(RVTableContext *context);
-R_API void r_anal_list_vtables(RAnal *anal, int rad);
+R_API char *r_anal_vtables_list(RAnal *anal, int rad);
 
 /* rtti */
 R_API char *r_anal_rtti_msvc_demangle_class_name(RVTableContext *context, const char *name);
@@ -1471,6 +1553,7 @@ R_API void r_anal_rtti_print_at_vtable(RAnal *anal, ut64 addr, int mode);
 R_API void r_anal_rtti_print_all(RAnal *anal, int mode);
 R_API void r_anal_rtti_recover_all(RAnal *anal);
 
+
 R_API RList *r_anal_preludes(RAnal *anal);
 R_API bool r_anal_is_prelude(RAnal *anal, ut64 addr, const ut8 *data, int len);
 
@@ -1502,6 +1585,15 @@ typedef enum {
 	R_ANAL_CLASS_ERR_OTHER
 } RAnalClassErr;
 
+// Forward declarations for fini functions
+R_API void r_anal_class_method_fini(RAnalMethod *meth);
+R_API void r_anal_class_base_fini(RAnalBaseClass *base);
+R_API void r_anal_class_vtable_fini(RAnalVTable *vtable);
+
+R_VEC_TYPE_WITH_FINI (RVecAnalMethod, RAnalMethod, r_anal_class_method_fini);
+R_VEC_TYPE_WITH_FINI (RVecAnalBaseClass, RAnalBaseClass, r_anal_class_base_fini);
+R_VEC_TYPE_WITH_FINI (RVecAnalVTable, RAnalVTable, r_anal_class_vtable_fini);
+
 /* c */
 R_API char *r_anal_cparse(RAnal *anal, const char *code, char **error_msg);
 R_API char *r_anal_cparse_file(RAnal *anal, const char *path, const char *dir, char **error_msg);
@@ -1513,31 +1605,28 @@ R_API SdbList *r_anal_class_get_all(RAnal *anal, bool sorted);
 R_API void r_anal_class_foreach(RAnal *anal, SdbForeachCallback cb, void *user);
 R_API RAnalClassErr r_anal_class_rename(RAnal *anal, const char *old_name, const char *new_name);
 
-R_API void r_anal_class_method_fini(RAnalMethod *meth);
 R_API RAnalClassErr r_anal_class_method_get(RAnal *anal, const char *class_name, const char *meth_name, RAnalMethod *meth);
-R_API RVector/*<RAnalMethod>*/ *r_anal_class_method_get_all(RAnal *anal, const char *class_name);
+R_API RVecAnalMethod *r_anal_class_method_get_all(RAnal *anal, const char *class_name);
 R_API RAnalClassErr r_anal_class_method_set(RAnal *anal, const char *class_name, RAnalMethod *meth);
 R_API RAnalClassErr r_anal_class_method_rename(RAnal *anal, const char *class_name, const char *old_meth_name, const char *new_meth_name);
 R_API RAnalClassErr r_anal_class_method_delete(RAnal *anal, const char *class_name, const char *meth_name);
 
-R_API void r_anal_class_base_fini(RAnalBaseClass *base);
 R_API RAnalClassErr r_anal_class_base_get(RAnal *anal, const char *class_name, const char *base_id, RAnalBaseClass *base);
-R_API RVector/*<RAnalBaseClass>*/ *r_anal_class_base_get_all(RAnal *anal, const char *class_name);
+R_API RVecAnalBaseClass *r_anal_class_base_get_all(RAnal *anal, const char *class_name);
 R_API RAnalClassErr r_anal_class_base_set(RAnal *anal, const char *class_name, RAnalBaseClass *base);
 R_API RAnalClassErr r_anal_class_base_delete(RAnal *anal, const char *class_name, const char *base_id);
 
-R_API void r_anal_class_vtable_fini(RAnalVTable *vtable);
 R_API RAnalClassErr r_anal_class_vtable_get(RAnal *anal, const char *class_name, const char *vtable_id, RAnalVTable *vtable);
-R_API RVector/*<RAnalVTable>*/ *r_anal_class_vtable_get_all(RAnal *anal, const char *class_name);
+R_API RVecAnalVTable *r_anal_class_vtable_get_all(RAnal *anal, const char *class_name);
 R_API RAnalClassErr r_anal_class_vtable_set(RAnal *anal, const char *class_name, RAnalVTable *vtable);
 R_API RAnalClassErr r_anal_class_vtable_delete(RAnal *anal, const char *class_name, const char *vtable_id);
 
-R_API void r_anal_class_print(RAnal *anal, const char *class_name, bool detailed);
+R_API char *r_anal_class_print(RAnal *anal, const char *class_name, bool detailed);
 R_API void r_anal_class_json(RAnal *anal, PJ *j, const char *class_name);
-R_API void r_anal_class_list(RAnal *anal, int mode);
-R_API void r_anal_class_list_bases(RAnal *anal, const char *class_name);
-R_API void r_anal_class_list_vtables(RAnal *anal, const char *class_name);
-R_API void r_anal_class_list_vtable_offset_functions(RAnal *anal, const char *class_name, ut64 offset);
+R_API char *r_anal_class_list(RAnal *anal, int mode);
+R_API char *r_anal_class_list_bases(RAnal *anal, const char *class_name);
+R_API char *r_anal_class_list_vtables(RAnal *anal, const char *class_name);
+R_API char *r_anal_class_list_vtable_offset_functions(RAnal *anal, const char *class_name, ut64 offset);
 R_API RGraph/*<RGraphNodeInfo>*/ *r_anal_class_get_inheritance_graph(RAnal *anal);
 
 R_API RAnalEsilCFG *r_anal_esil_cfg_new(void);
@@ -1559,7 +1648,7 @@ R_API bool r_anal_esil_dfg_reg_is_const(RAnalEsilDFG *dfg, const char *reg);
 R_API RList *r_anal_types_from_fcn(RAnal *anal, RAnalFunction *fcn);
 
 R_API RAnalBaseType *r_anal_get_base_type(RAnal *anal, const char *name);
-R_API void r_parse_pdb_types(const RAnal *anal, const RPdb *pdb);
+R_API void r_parse_pdb_types(const RAnal *anal, const RBinPdb *pdb);
 R_API void r_anal_save_base_type(const RAnal *anal, const RAnalBaseType *type);
 R_API void r_anal_base_type_free(RAnalBaseType *type);
 R_API RAnalBaseType *r_anal_base_type_new(RAnalBaseTypeKind kind);
@@ -1591,8 +1680,13 @@ R_API void r_anal_backtrace_fini(RAnal *a);
 R_API void r_anal_backtrace_list(RAnal *a, ut64 addr, int opt);
 
 /* plugin pointers */
+extern RAnalPlugin r_anal_plugin_blaze;
 extern RAnalPlugin r_anal_plugin_null;
 extern RAnalPlugin r_anal_plugin_a2f;
+extern RAnalPlugin r_anal_plugin_path;
+extern RAnalPlugin r_anal_plugin_sbpf;
+extern RAnalPlugin r_anal_plugin_tcc;
+extern RAnalPlugin r_anal_plugin_tp;
 
 #ifdef __cplusplus
 }
