@@ -7,7 +7,7 @@
 #include <r_util.h>
 #include <r_lib.h>
 #include <r_socket.h>
-#include <r_vector.h>
+#include <r_vec.h>
 #include <r_util/r_w32dw.h>
 
 #define R_IO_SEEK_SET 0
@@ -103,7 +103,6 @@ typedef struct r_io_undo_w_t {
 	size_t len; /* length */
 } RIOUndoWrite;
 
-// io cache
 typedef struct io_cache_item_t {
 	RInterval *tree_itv;
 	RInterval itv;
@@ -112,12 +111,15 @@ typedef struct io_cache_item_t {
 	bool written;
 } RIOCacheItem;
 
+R_VEC_TYPE (RVecRIOCacheItemPtr, RIOCacheItem *);
+
+// io cache
 typedef struct r_io_cache_layer_t {
 #if 0
 	char *name;
 	int fd, bd, ... :?
 #endif
-	RPVector *vec; // a vector of items
+	RVecRIOCacheItemPtr *vec; // a vector of items
 	RRBTree *tree; // faster access to the items
 	// RRBComparator ci_cmp_cb; // this comparator can be inside the rbtree impl
 } RIOCacheLayer;
@@ -339,6 +341,7 @@ typedef RIOMap *(*RIOMapGetAt)(RIO *io, ut64 addr);
 typedef RIOMap *(*RIOMapGetPaddr)(RIO *io, ut64 paddr);
 typedef bool (*RIOAddrIsMapped)(RIO *io, ut64 addr);
 typedef RIOMap *(*RIOMapAdd)(RIO *io, int fd, int flags, ut64 delta, ut64 addr, ut64 size);
+typedef bool (*RIOMapLocate)(RIO *io, ut64 *addr, const ut64 size, ut64 load_align);
 #if HAVE_PTRACE
 typedef long (*RIOPtraceFn)(RIO *io, r_ptrace_request_t request, pid_t pid, void *addr, r_ptrace_data_t data);
 typedef void *(*RIOPtraceFuncFn)(RIO *io, void *(*func)(void *), void *user);
@@ -378,6 +381,7 @@ typedef struct r_io_bind_t {
 	RIOMapGetAt map_get_at;
 	RIOMapGetPaddr map_get_paddr;
 	RIOMapAdd map_add;
+	RIOMapLocate map_locate;
 	RIOV2P v2p;
 	RIOP2V p2v;
 #if HAVE_PTRACE
@@ -682,6 +686,7 @@ extern RIOPlugin r_io_plugin_pvm;
 extern RIOPlugin r_io_plugin_serial;
 extern RIOPlugin r_io_plugin_cyclic;
 extern RIOPlugin r_io_plugin_uf2;
+extern RIOPlugin r_io_plugin_tap;
 
 #if __cplusplus
 }
